# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module Bouchot
##   Classe gnrique bouchot - contenant un ou plusieurs modules
##   tels que Tribune, News, etc.
## ----------------------------------------------------------------------

import os, os.path
from Tkinter import *
import Pmw
import logging
log = logging.getLogger('pycc')

from constants import *
from coinlib.utils import *
from coinlib.persistants import PinniStyleSet, OptionSet, Option


class BouchotOptions(OptionSet):
  """ Options gnrales d'un bouchot
  """
  def setObjects(self):
    self._objects = {
      'http_server':     Option("Adresse du serveur http"),
      'bouchot_aliases': Option("Aliases"),
      'default_id':      Option("Identit par dfaut", "", TYPE_CHOICE),
      'startup_modules': Option("Modules  activer au lancement", [], TYPE_CHOICE_MULTI),
      'base_modules':    Option("Modules de base", {'tribune': 'tribune'}),
      'toedit':          Option("Editables", {
          "Gnral": [ 'http_server', 'bouchot_aliases', 'default_id', 'startup_modules' ] } )
    }
  

class Bouchot(object):
  
  def __init__(self, pycc, name):
    log.debug("## Entering Bouchot.__init__(%s)" % name)
    self.pycc = pycc
    self.name = name
    self.init()
  
  def init(self):
    log.debug("## Entering Bouchot.init")
    self.config_path = os.path.join(self.pycc.config_path, "bouchots", self.name)
    self.started = 0
    self.blinking = 0
    self.cur_color = self.pycc.system_color
    self.modules = {}
    self.trib = None
    self.load_config()
    self.load_modules()
  
  def load_config(self):
    log.debug("## Entering Bouchot.load_config")
    self.styles = PinniStyleSet(path=self.config_path)
    self.options = BouchotOptions(path=self.config_path)
  
  def apply_config(self):
    tostart = [mod for mod in self.modules.keys() if mod in self.options.val('startup_modules')]
    for mod in tostart:
      self.start_module(mod)
    tostop = [mod for mod in self.modules.keys() if mod not in self.options.val('startup_modules')]
    for mod in tostop:
      self.stop_module(mod)
  
  def save_config(self):
    log.debug("## Entering Bouchot.save_config")
    self.styles.save()
    self.options.save()
    for mod in self.modules.values():
      mod.save_config()
  
  def apply_styles(self):
    for module in self.modules.values():
      if module.loaded:
        if module.viewer_open:
          module.apply_styles()
  
  def build_interface(self, master):
    log.debug("## Entering Bouchot.build_interface")
    self.root = master
    self.pw = Pmw.PanedWidget(master, orient='vertical')
    self.pw.pack(expand=1, fill='both')
    self.ledbar = Frame(master)
    self.ledbar.pack(fill='x')
  
  def new_module(self, name, type):
    log.debug("## Entering Bouchot.new_module(%s, %s)" % (name, type))
    if name in get_modules(self.pycc.config_path, self.name):
      log.error("%s: Impossible de crer le module '%s': module dj existant" % (self.name, name))
      return 0
    if type not in self.pycc.modules.keys():
      log.error("%s: Impossible de crer le module '%s': type de module '%s' inconnu" % (self.name, name, type))
      return 0
    try:
      os.mkdir(os.path.join(self.config_path, name))
      f = open(os.path.join(self.config_path, name, "mod.type"), "w")
      f.write("%s\n" % type)
      f.close()
    except IOError, value:
      log.error("%s: Impossible de crer le module '%s': %s" % (self.name, name, value))
      return 0
    # self.load_module(name)
    return 1
  
  def destroy_module(self, name):
    log.debug("%s: destruction du module '%s'" % (self.name, name))
    self.stop_module(name)
    try:
      top = os.path.join(self.config_path, name)
      for root, dirs, files in os.walk(top, topdown=False):
        for fich in files:
          os.remove(os.path.join(root, fich))
        for fich in dirs:
          os.rmdir(os.path.join(root, fich))
      os.rmdir(top)
    except:
      log.error("%s: Impossible de dtruire tous les fichiers du module '%s'" % (self.name, name))
    del self.modules[name]
  
  def load_module(self, mod):
    log.debug("## Entering Bouchot.load_module(%s)" % mod)
    try:
      f = open(os.path.join(self.config_path, mod, "mod.type"), "r")
      modtype = f.read().split()[0]
      f.close()
    except:
      log.error("%s: Impossible de trouver le type du module '%s'" % (self.name, mod))
      return
    #try:
    self.modules[mod] = self.pycc.modules[modtype](self, mod)
    self.options.val('base_modules')[mod] = modtype
    # self.modules[mod].type = modtype
    if mod == "tribune":
      self.trib = self.modules[mod]
    #except:
    #  log.error("%s:%s: Impossible de construire un module '%s'" % (self.name, mod, modtype))
  
  def load_modules(self):
    log.debug("## Entering Bouchot.load_modules")
    self.modules = {}
    for mod in get_modules(self.pycc.config_path, self.name):
      self.load_module(mod)
    log.info("%s: modules chargs: %s" % (self.name, self.modules.keys()))
  
  def start_module(self, mod):
    log.info("%s: Dmarrage du module %s" % (self.name, mod))
    module = self.modules[mod]
    if module.loaded:
      log.info("%s: Module %s dj dmarr" % (self.name, mod))
      return
    module.initialize()
    module.start()
    if mod == "tribune":
      self.trib = module
  
  def stop_module(self, mod):
    log.info("%s: Quit module %s" % (self.name, mod))
    self.modules[mod].quit()
    if mod == "tribune":
      self.trib = None
  
  def start(self):
    log.debug("## Entering Bouchot.start")
    modlist = [mod for mod in self.modules.keys() if mod in self.options.val('startup_modules')]
    for mod in modlist:
      self.start_module(mod)
    self.started = 1
    self.options.set('startup_modules', modlist)
  
  def quit(self):
    log.debug("## Entering Bouchot.quit")
    for mod in self.modules.keys():
      self.stop_module(mod)
    self.started = 0
  
  def setState(self, state):
    log.debug("## Entering Bouchot.setState(%s)" % state)
    if not self.started: return
    current = self.pycc.get_bouchot()
    if current is not None and current != self.name:
      self.pycc.colorize_tab(self.name, state)
      # self.blinking = 0
  
  def blink(self, color=STATE_BLINK, delay=500):
    if self.blinking: return
    current = self.pycc.get_bouchot()
    if current is not None and current != self.name:
      self.blinking = 1
      self.pycc.blink_tab(self.name, color, delay)
  
  def stop_blink(self):
    self.blinking = 0
  
  def getCurId(self):
    myid = self.options.val('default_id')
    return self.pycc.identities.get(myid)
  