# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module cointklib.widgets
##   Widgets avancs spcifiques  pycc
## ----------------------------------------------------------------------

from Tkinter import *
import tkMessageBox
import Pmw
import logging
import sys, os, os.path

sys.path.append('..')

from coinlib.utils import *
from coinlib.persistants import *
from basicwidgets import *
from dialogs import *

log = logging.getLogger('pycc')

## -----------------------------------------------
## Classe gnrique
## -----------------------------------------------


class CoinObjectEditor(Frame, Pmw.ScrolledFrame):
  
  def __init__(self, parent, object, extra_arg=None, scrolled=0, **kwargs):
    self.parent = parent
    self.object = object
    self.extra_arg = extra_arg
    if scrolled:
      Pmw.ScrolledFrame.__init__(self, parent)
      self._fr = self.interior()
      # self._fr.config(height=10, width=10) # Pour viter un sizing trop grand par dfaut
      self._fr.config(**kwargs)
    else:
      Frame.__init__(self, parent, **kwargs)
      self._fr = self
    self.body()
  
  def body(self):
    """Construction des widgets composant l'diteur"""
    pass
  
  def reset(self):
    """Mthode de rinitialisation avec les valeurs par dfaut"""
    pass
  
  def apply(self):
    """Sauvegarde des valeurs entres dans l'interface vers l'objet"""
    pass
  
  def get(self):
    self.apply()
    return self.object
  

## -----------------------------------------------
## Editeur de styles
## -----------------------------------------------

class StyleEditor(CoinObjectEditor):

  params = [ 'font', 'background', 'foreground', 'underline', 'overstrike', 'relief', 'borderwidth' ]
  
  def __init__(self, parent, object, show_default=0, **kwargs):
    self.show_default = show_default
    CoinObjectEditor.__init__(self, parent, object, **kwargs)
  
  def body(self):
    self.widgets = {}
    if self.show_default:
      self.wuse_default = BoolField(self._fr, "Utiliser l'apparence par dfaut", self.object.use_default)
      self.wuse_default.pack(side=TOP, pady=2, anchor=W, fill=X)
    fr = Frame(self._fr, border=2, relief='groove')
    self.widgets['font'] = FontField(fr, self.object.get('font'))
    self.widgets['font'].pack(side=TOP, padx=4, pady=2, anchor=E)
    frb = Frame(fr)
    self.widgets['underline'] = BoolField(frb, "Soulign", self.object.get('underline'))
    self.widgets['underline'].pack(side=LEFT, anchor=W)
    self.widgets['overstrike'] = BoolField(frb, "Barr", self.object.get('overstrike'))
    self.widgets['overstrike'].pack(side=LEFT, anchor=W)
    frb.pack(side=TOP, padx=4, pady=2, anchor=W)
    self.widgets['foreground'] = ColorField(fr, "Couleur d'avant-plan", self.object.get('foreground'))
    self.widgets['foreground'].pack(side=TOP, padx=4, pady=2, anchor=E)
    self.widgets['background'] = ColorField(fr, "Couleur d'arrire-plan", self.object.get('background'))
    self.widgets['background'].pack(side=TOP, padx=4, pady=2, anchor=E)
    self.widgets['relief'] = ComboField(fr, "Type de bordure", ["", "raised", "sunken", "ridge", "groove"], self.object.get('relief'))
    self.widgets['relief'].pack(side=TOP, padx=4, pady=2, anchor=W)
    self.widgets['borderwidth'] = IntField(fr, "Epaisseur de la bordure (en pixels)", self.object.get('borderwidth'))
    self.widgets['borderwidth'].pack(side=TOP, padx=4, pady=2, anchor=E)
    fr.pack(anchor=W, pady=2)
  
  def bind_onchange(self, callback):
    for param in self.params:
      self.widgets[param].bind_onchange(callback)
    if self.show_default:
      self.wuse_default.bind_onchange(callback)
  
  def get_style(self):
    res = ""
    for param in self.params:
      val = self.widgets[param].get()
      if val:
        if type(val) == type(""):
          val = "'"+val+"'"
        res += "%s=%s, " % (param, val)
    return res[:-2]
  
  def use_default(self):
    return self.show_default and self.wuse_default.get()
  
  def apply(self):
    if self.show_default:
      self.object.use_default = self.use_default()
    for param in self.params:
      value = self.widgets[param].get()
      if value: 
        self.object.set(param, value)
      else:
        self.object.delete(param)
  
class ThemeEditor(CoinObjectEditor):
  
  def __init__(self, parent, object, theme_path='.', mode='pinni', show_default=0, default_styleset=None, **kwargs):
    self.theme_path = os.path.join(theme_path, mode)
    self.mode = mode
    self.show_default = show_default
    self.default_styleset = default_styleset
    self.theme_list = ["<actuel>"] + [fich[:-4] for fich in os.listdir(self.theme_path) if fich[-4:] == ".xml"]
    if self.show_default: self.theme_list[:0] = ["<apparence par dfaut>"]
    CoinObjectEditor.__init__(self, parent, object, **kwargs)
  
  def body(self):
    self.styleset = self.object.get()
    Label(self._fr, background=COLOR_TITLE, text="Editeur de Styles").pack(anchor=NW, fill=X)
    self.fr = Frame(self._fr)
    frlist = Frame(self.fr)
    self.style_list = SortedListBox(frlist, self.styleset, width=30, height=12)
    self.style_list.pack(anchor=NW, fill=Y, expand=1)
    self.style_list.bind_select(self.select_style)
    frbut = Frame(frlist)
    Button(frbut, text="Nouveau style...", border=1, command=self.new_style).pack(side=LEFT)
    Button(frbut, text="Supprimer style", border=1, command=self.del_style).pack(side=RIGHT)
    frbut.pack()
    frlist.pack(side=LEFT, anchor=NW, fill=Y, expand=1)
    self.fr.pack(anchor=NW, fill=Y, expand=1)
    frtheme = Frame(self._fr, border=2, relief='groove')
    self.wtheme_list = ComboField(frtheme, "Thme", self.theme_list, value="<actuel>", width=20)
    self.wtheme_list.bind_onchange(self.load_theme)
    self.wtheme_list.pack(side=LEFT, pady=2)
    Button(frtheme, text="Enregistrer les modifications", command=self.save_theme).pack(side=LEFT, pady=2)
    Button(frtheme, text="Nouveau thme...", command=self.new_theme).pack(side=LEFT, pady=2)
    frtheme.pack(fill=X, expand=1, pady=2, padx=2)
    Label(self._fr, text="Aperu").pack(side=LEFT)
    self.apercu = Text(self._fr, wrap=WORD, width=72, height=7)
    self.apercu.bind("<1>", self.click_apercu)
    self.apercu.config(cursor="arrow")
    self.apercu.pack(fill=BOTH, expand=1)
    self.wstyle = {}
    self.fill_apercu()
    self.fill_list()
  
  def fill_list(self):
    if len(self.wstyle) > 0:
      for styleditor in self.wstyle.values():
        styleditor.destroy()
    self.wstyle = {}
    for name, style in self.styleset.items():
      self.wstyle[name] = StyleEditor(self.fr, style, self.show_default)
      self.wstyle[name].bind_onchange(self.update_apercu)
    self.style_list.select('base')
    self.update_apercu()
  
  def apply(self):
    for name, styleditor in self.wstyle.items():
      styleditor.apply()
      self.styleset[name] = styleditor.object
    stl_to_remove = [ name for name in self.styleset.keys() if name not in self.wstyle.keys() ]
    for name in stl_to_remove:
      del self.styleset[name]
    self.object.save()
  
  def click_apercu(self, event):
    tags = [ tag for tag in self.apercu.tag_names("@%d,%d" % (event.x, event.y)) if tag not in ['b', 'bi', 'i', 'u', 's'] ]
    if not tags:
      self.style_list.select('base')
    else:
      # print "-", tags
      self.style_list.select(tags[-1])
  
  def select_style(self):
    for name, fstyle in self.wstyle.items():
      fstyle.pack_forget()
    stl = self.style_list.get()
    if not stl: return
    self.wstyle[stl].pack(side=LEFT, anchor=NW, fill=BOTH, expand=1, padx=5, pady=5)
  
  def new_style(self):
    idx = 1 + max( [0] + [ int(name[6:]) for name in self.wstyle.keys() if name[:6] == "custom" ] )
    res = tkSimpleDialog.askstring("DaStyleManager - Nouveau style", \
            "Description:", initialvalue="Mon style %d" % idx, parent=self)
    if res is None: return
    newstl = "custom%d" % idx
    style = Style("[custom] " + res)
    self.wstyle[newstl] = StyleEditor(self.fr, style)
    self.style_list.add(newstl, style)
    self.style_list.select(newstl)
  
  def del_style(self):
    stl = self.style_list.get()
    if stl is None: return 0
    if stl[:6] != "custom":
      tkMessageBox.showwarning("DaStyleManager", \
        "Ce style ne peut tre supprim, il fait partie des styles de base de pyCoinCoin", \
        parent=self)
      return 0
    self.wstyle[stl].pack_forget()
    del self.wstyle[stl]
    self.style_list.remove(stl)
  
  def new_theme(self):
    res = tkSimpleDialog.askstring("DaStyleManager - Nouveau thme", \
            "Nom du thme:", initialvalue="mon_theme", parent=self)
    if res is None: return
    if res in self.theme_list:
      tkMessageBox.showwarning("DaStyleManager", \
        "Impossible de crer le thme '%s':\nIl existe dj un thme de ce nom" % res, parent=self)
      return
    self.theme_list.append(res)
    self.wtheme_list.setitems(self.theme_list)
    self.wtheme_list.set(res)
    self.save_theme()
  
  def load_theme(self):
    name = self.wtheme_list.get()
    if name == "<actuel>": return
    if name == "<apparence par dfaut>":
      for style in self.wstyle.values():
        style.wuse_default.set(1)
      self.update_apercu()
      return
    theme = Theme(name, path=self.theme_path)
    if not theme.load():
      tkMessageBox.showwarning("DaStyleManager", \
        "Impossible de charger ce thme...\nVrifiez que le fichier '%s.xml' existe bien" % name, \
        parent=self)
      self.wtheme_list.set("<actuel>")
      return
    theme.copy_styles_to(self.object)
    self.style_list.items = self.object.get()
    self.style_list.update()
    self.fill_list()
  
  def save_theme(self):
    name = self.wtheme_list.get()
    if name == "<actuel>": return
    self.apply()
    theme = Theme(name, path=self.theme_path)
    theme.import_styles_from(self.object)
    theme.save()
  
  def update_apercu(self, event=None):
    for name, style in self.wstyle.items():
      if self.show_default and style.use_default():
        try:
          curstyle = self.default_styleset.get(name)
        except:
          curstyle = style # cas d'un style non dfini dans les dfauts
      else:
        curstyle = style
      try:
        if name == 'base':
          eval("self.apercu.config(%s)" % curstyle.get_style())
          basefont = style.widgets['font'].get()
        else:
          bufidx = self.apercu.tag_ranges(name)
          self.apercu.tag_delete(name)
          eval("self.apercu.tag_config('%s', %s)" % (name, curstyle.get_style()))
          for cpt in range(0, len(bufidx), 2):
            self.apercu.tag_add(name, bufidx[cpt], bufidx[cpt+1])
      except:
        log.warning("StyleEditor: Gne gna du mal avec le style '%s'" % name)
    try:
      self.apercu.tag_config("b",  font=(basefont[0], basefont[1], 'bold'))
      self.apercu.tag_config("i",  font=(basefont[0], basefont[1], 'italic'))
      self.apercu.tag_config("bi", font=(basefont[0], basefont[1], 'bold italic'))
    except:
      log.warning("StyleEditor: Bigre, je ne parviens pas  configurer la police du style de base")
    self.apercu.tag_config("u", underline=1)
    self.apercu.tag_config("s", overstrike=1)
    sort_tags(self.apercu, self.mode)
  
  def fill_apercu(self):
    if self.mode == 'pinni':
      text = "15:40:23 gruik Plop les moules<\n" + \
             "15:42:42 IE6/WinXP 15:40:23 hi kevin sa fartent ? Arf trop trop fort [url] ! Hein quoi c'est dj pass ? :)\n" + \
             "15:45:00 monlogin 14:00@dlfp plodocus ?\n" + \
             "15:46:10 UnQuiRepond 15:45 [url] fortune detected || et parfois aussi, ton clavier se blo\n" + \
             "15:46:20 pika 15:42:42 [:totoz]"
      tags = { 'time':  [(0,8), (32,40), (141,149), (181,189), (271,279)],
               'login': [(9,14), (150,159), (190,201), (280,284)],
               'info':  [(41,50)],
               'a':          [(101,106), (208,213)],
               'b':          [(101,106), (208,213)],
               'urlvisit':   [(101,106)],
               'urlhilight': [(294,302)],
               'clock':      [(51,59), (202,207), (285,293)],
               'crossclock': [(159,169)],
               'hilightref': [(51,59)],
               'tt':      [(214,230)],
               'hilight': [(0,32)],
               'myposts': [(141,149)],
               'answer':  [(181,189)] }
    else:
      text = "14:00 nosaure ? ====> <b>Moment fortune http://quadaemon.free.fr/tribune.fortune</b> <==== || " + \
             "15:42:42@woof je suis pas un faique, %l c'est moi ! Argh j'cris trop, a dpasse la taille maxi"
      tags = { 'horloge': [(0,5), (94,107)],
               'balise':  [(22,25), (80,84)],
               'url':     [(40,80)],
               'spec':    [(131,133)],
               'depasse': [(165,191)]  }
    self.apercu.config(state='normal')
    self.apercu.delete(0.0, 'end')
    self.apercu.insert('end', text)
    for tag, tagranges in tags.items():
      for start, end in tagranges:
        self.apercu.tag_add(tag, "0.0+%dc" % start, "0.0+%dc" % end)
    self.apercu.config(state='disabled')
  

## -----------------------------------------------
## Editeur d'identits
## -----------------------------------------------

class IdentityEditor(CoinObjectEditor):
  
  def body(self):
    self.trib_list = {}    # Hash nom => objet tribune
    self.login = Label(self._fr, foreground="#008000")
    self.login.pack(anchor=W)
    self.ua = TextField(self._fr, 'User agent')
    self.ua.pack(anchor=E)
    Label(self._fr, text="Cookies:").pack(anchor=W)
    self.fr = Frame(self._fr, border=1, relief=RIDGE)
    self.cookies = {}
    self.del_cookies = {}
    self.grab_cookies = {}
    self.fr.pack(padx=2, pady=2)
    Button(self._fr, text="Dfinir nouveau cookie...", border=1, command=self.new_cookie).pack(anchor=E)
    # Button(self._fr, text="Se loguer sur un bouchot...", border=1, command=self.logon).pack(anchor=E)
    self.reset()
  
  def set_trib_list(self, list):
    self.trib_list = list
  
  def new_cookie(self):
    self.selected = None
    SelectDialog(self, self.trib_list.keys(), "IdentityEditor", \
      "Choisir le bouchot sur lequel vous voulez dfinir un nouveau cookie", \
      width=40, height=8)
    if self.selected is not None:
      trib = self.selected
      if trib in self.cookies:
        tkMessageBox.showinfo("IdentityEditor", \
          "Ce bouchot est dj dans votre liste.\nPour y dfinir un autre cookie, il suffit de l'ajouter  la suite des autres, spar par '; '.", \
          parent=self)
      else:
        cookie = ''
        if self.trib_list[trib] is not None:
          cookie = self.trib_list[trib].options.val('cookie_scheme')
        self.add_cookie(trib, cookie)
      self.cookies[trib].focus_set()
  
  def add_cookie(self, trib, cookie=''):
    self.cookies[trib] = TextField(self.fr, trib, cookie)
    self.del_cookies[trib] = Button(self.cookies[trib], text="Del", border=1, command=lambda s=self, t=trib: s.remove_cookie(t))
    self.del_cookies[trib].pack(side=RIGHT)
    self.grab_cookies[trib] = Button(self.cookies[trib], text="Grab", border=1, command=lambda s=self, t=trib: s.grab_cookie(t))
    self.grab_cookies[trib].pack(side=RIGHT)
    self.cookies[trib].pack(anchor=E)
  
  def remove_cookie(self, trib):
    self.cookies[trib].destroy()
    self.del_cookies[trib].destroy()
    self.grab_cookies[trib].destroy()
    del self.cookies[trib]
    del self.del_cookies[trib]
    del self.grab_cookies[trib]
  
  def grab_cookie(self, trib):
    if self.trib_list[trib] is None:
      return
    cookies = self.trib_list[trib].grab_cookies()
    if not cookies:
      tkMessageBox.showinfo("IdentityEditor", \
          "Aucun cookie trouv, daisolai", \
          parent=self)
      return
    self.cookies[trib].set(cookies)
  
  def reset(self):
    self.login.config(text = "Login/pseudo: "+self.object.login)
    self.ua.set(self.object.ua)
    for trib in self.cookies.keys():
      self.remove_cookie(trib)
    for trib, cookie in self.object.cookies.items():
      self.add_cookie(trib, cookie)
  
  def apply(self):
    self.object.ua = self.ua.get()
    self.object.cookies = {}
    for trib, cookie in self.cookies.items():
      self.object.cookies[trib] = cookie.get()
  

class IdentitiesEditor(CoinObjectEditor):
  
  def body(self):
    Label(self._fr, text="Liste des identits", background=COLOR_TITLE).pack(fill=X, anchor=NW)
    self.fr = Frame(self._fr)
    ifr = Frame(self._fr)
    self.idlist = SortedListBox(ifr, self.object.get(), height=8, width=20)
    self.idlist.pack(side=TOP, fill=BOTH, expand=1, anchor=NW)
    self.idlist.bind_select(self.select)
    bbox = Frame(ifr)
    Button(bbox, text="Nouvelle identit...", border=1, command=self.new).pack(side=LEFT, anchor=W)
    Button(bbox, text="Supprimer", border=1, command=self.delete).pack(side=RIGHT, anchor=E)
    bbox.pack(fill=X)
    lbox = Frame(ifr)
    Button(lbox, text="Log on bouchot...", border=1, command=self.logon).pack(side=LEFT, anchor=W)
    Button(lbox, text="Logout from bouchot...", border=1, command=self.logout).pack(side=RIGHT, anchor=E)
    lbox.pack(fill=X)
    ifr.pack(side=LEFT, anchor=NW, fill=Y, expand=1)
    self.ids = {}
    for id in self.object.get().keys():
      self.ids[id] = IdentityEditor(self.fr, self.object.get(id))
    self.fr.pack(anchor=NW)
  
  def set_trib_list(self, list):
    self.trib_list = list
    for idf in self.ids.values():
      idf.set_trib_list(self.trib_list)
  
  def select(self):
    for id in self.ids.keys():
      self.ids[id].pack_forget()
    cur_id = self.idlist.get()
    if not cur_id: return
    self.ids[cur_id].pack(side=LEFT, padx=5, pady=5, anchor=NW)
  
  def new(self):
    res = tkSimpleDialog.askstring("pyCoinCoin - Nouvelle identit", \
            "Login/Pseudo:", parent=self)
    if res is not None:
      ident = Identity(res)
      self.ids[res] = IdentityEditor(self.fr, ident)
      self.ids[res].set_trib_list(self.trib_list)
      self.idlist.add(res, ident)
      self.idlist.select(res)
  
  def delete(self):
    id = self.idlist.get()
    if id is None: return 0
    if not tkMessageBox.askyesno("IdentityEditor", "Confirmez-vous la suppression de l'identit " + \
      "'%s' ?\nNB: toute donne associe (cookie, etc.) sera perdue." % id, parent=self):
        return 0
    self.ids[id].pack_forget()
    del self.ids[id]
    self.idlist.remove(id)
  
  def logon(self):
    id = self.idlist.get()
    pycc = self.trib_list.values()[0].bouchot.pycc
    trib = pycc.get_bouchot()
    self.result = ("", "")
    LoginDialog(self, pycc, trib, id)
    id, trib = self.result
    if id:
      self.ids[id].reset()
  
  def logout(self):
    id = self.idlist.get()
    pycc = self.trib_list.values()[0].bouchot.pycc
    trib = pycc.get_bouchot()
    self.result = ("", "")
    LogoutDialog(self, pycc, trib, id)
    id, trib = self.result
    if id:
      self.ids[id].reset()
  
  def apply(self):
    for id in self.ids.keys():
      self.ids[id].apply()
    # MAJ et ajout des nouvelles identits ventuelles
    for id in self.ids.keys():
      self.object.add(id, self.ids[id].get())
    # Effaage effectif des identits ventuellement effaces
    ids_toremove = [ id for id in self.object.get().keys() if id not in self.ids.keys() ]
    for id in ids_toremove:
      self.object.delete(id)
    self.object.save()
  

## -----------------------------------------------
## Editeur d'actions
## -----------------------------------------------

class ActionDialog(CoinDialog):
  
  def __init__(self, parent, object, extra_arg):
    self.object = object
    self.extra_arg = extra_arg
    CoinDialog.__init__(self, parent, "Paramtres de l'action", [BUT_OK, BUT_CANCEL], modal=1)
  
  def body(self, master):
    Label(master, text=self.object.descr(), background=COLOR_TITLE).pack(fill=X)
    self.w_action = ActionEditor(master, self.object, extra_arg=self.extra_arg)
    self.w_action.pack()
  
  def apply(self):
    self.w_action.apply()
  
class ActionEditor(CoinObjectEditor):
  
  def body(self):
    # extra_arg contient le contexte (PINNI|PALMI) + une ref vers pycc
    ctxt, pycc = self.extra_arg
    # Label(self._fr, text=actions_descr[self.object.type], background=COLOR_TITLE, font="Helvetica 9 bold").pack(fill=X)
    self.w_param = None
    if self.object.type == ACTION_POST:
      if ctxt != PALMI:
        self.w_param = TextField(self._fr, "Message", self.object.param)
        self.w_param.pack(fill=X, anchor=E)
      
      self.w_ano = BoolField(self._fr, "Anonyme", not self.object.auth)
      self.w_ano.pack(fill=X, anchor=W)
      
      self.w_multi = BoolField(self._fr, "Multipost (sur toutes les tribunes actives)", self.object.multi)
      self.w_multi.pack(fill=X, anchor=W)
      
      self.w_trib = ComboField(self._fr, "Tribune destinataire", ["<par dfaut>"]+[t for t, b in pycc.bouchots.items() if b.trib is not None])
      strib = self.object.tribdest
      if strib is None:
        strib = "<par dfaut>"
      self.w_trib.set(strib)
      self.w_trib.pack(fill=X, anchor=W)
      
      self.w_id = ComboField(self._fr, "Avec l'identit", ["<par dfaut>"]+pycc.identities.get().keys())
      sid = self.object.iddest
      if sid is None:
        sid = "<par dfaut>"
      self.w_trib.set(sid)
      self.w_id.pack(fill=X, anchor=W)
    
    elif self.object.type == ACTION_STYLE:
      self.w_param = SortedListBox(self._fr, pycc.default_styles.get(), height=7, width=35)
      self.w_param.select(self.object.param)
      self.w_param.pack()
      self.w_target = ComboField(self._fr, "Appliquer  :", POST_FIELDS, self.object.target)
      self.w_target.pack(fill=X, anchor=W)
    
    elif self.object.type == ACTION_REMSTYLE:
      self.w_param = SortedListBox(self._fr, pycc.default_styles.get(), height=7, width=35)
      self.w_param.select(self.object.param)
      self.w_param.pack()
    
    elif self.object.type == ACTION_PLUGIN:
      self.w_param = SortedListBox(self._fr, pycc.plugins, height=7, width=30)
      self.w_param.select(self.object.param)
      self.w_param.pack()
    
    elif self.object.type == ACTION_MSG:
      self.w_param = TextField(self._fr, "Texte :", self.object.param)
      self.w_param.pack(fill=X, anchor=E)
      self.w_console = BoolField(self._fr, "Afficher dans la console", self.object.console)
      self.w_console.pack(fill=X, anchor=W)
      self.w_screen = BoolField(self._fr, "Afficher dans une fentre graphique", self.object.screen)
      self.w_screen.pack(fill=X, anchor=W)
    
    elif self.object.type == ACTION_BLINK:
      self.w_param = ColorField(self._fr, "Couleur :", self.object.param)
      self.w_param.pack(fill=X, anchor=E)
      self.w_delay = IntField(self._fr, "Vitesse du clignotement (en ms) :", self.object.delay)
      self.w_delay.pack(fill=X, anchor=E)
    
    elif self.object.type == ACTION_SOUND:
      self.w_param = FileField(self._fr, "Fichier :", self.object.param)
      self.w_param.pack(fill=X)
    
    elif self.object.type == ACTION_RESTAURE:
      self.w_param = BoolField(self._fr, "R-appliquer le post-processor", self.object.param)
      self.w_param.pack(fill=X, anchor=W)
    
    elif self.object.type == ACTION_REFRESHALL:
      pass
    
    else:
      if self.object.type == ACTION_SYSCMD:
        lbl = "Commande :"
      elif self.object.type == ACTION_TAGGER:
        lbl = "Balise :"
      elif self.object.type == ACTION_TEXT:
        lbl = "Texte :"
      else:
        lbl = "<Unknown action type>"
      self.w_param = TextField(self._fr, lbl, self.object.param)
      self.w_param.pack(fill=X)
    
    return self.w_param
  
  def apply(self):
    # AFR: des contrles sur les champs ?
    if self.w_param is not None:
      self.object.param = self.w_param.get()
    if self.object.type == ACTION_POST:
      self.object.auth = not self.w_ano.get()
      self.object.multi = self.w_multi.get()
      dtrib = self.w_trib.get()
      if dtrib == "<par dfaut>": dtrib = None
      self.object.tribdest = dtrib
      did = self.w_id.get()
      if did == "<par dfaut>": did = None
      self.object.iddest = did
    elif self.object.type == ACTION_STYLE:
      self.object.target = self.w_target.get()
    elif self.object.type == ACTION_MSG:
      self.object.console = self.w_console.get()
      self.object.screen = self.w_screen.get()
    elif self.object.type == ACTION_BLINK:
      self.object.delay = self.w_delay.get()


## -----------------------------------------------
## Editeur de conditions
## -----------------------------------------------

class CSetDialog(CoinDialog):
  
  def __init__(self, parent, object):
    self.object = object
    CoinDialog.__init__(self, parent, "Ensemble de conditions", [BUT_OK, BUT_CANCEL], modal=1)
  
  def body(self, master):
    self.w_cset = ConditionSetEditor(master, self.object)
    self.w_cset.pack()
  
  def apply(self):
    self.w_cset.apply()

class ConditionSetEditor(CoinObjectEditor):
  
  def body(self):
    contour = Pmw.Group(self._fr, tag_text="Conditions")
    contour.pack(fill=BOTH, padx=2, pady=2)
    fr = contour.interior()
    head = Frame(fr)
    Label(head, text=" Vrifier :", justify=LEFT, anchor=W).pack(side=LEFT, anchor=W)
    modes = [("toutes ces conditions", COND_ALL), ("une de ces conditions", COND_ANY)]
    self.type = IntVar()
    for text, mode in modes:
      b = Radiobutton(head, text=text, variable=self.type, value=mode)
      b.pack(side=LEFT, anchor=W)
    head.pack(side=TOP, fill=X, expand=1)
    self.csfr = Frame(fr)
    self.csfr.pack(side=TOP)
    foot = Frame(fr)
    Button(foot, width=10, text="Plus", command=self.addCond).pack(side=LEFT, padx=5, pady=5)
    Button(foot, width=10, text="Moins", command=self.delCond).pack(side=LEFT, padx=5, pady=5)
    foot.pack(side=TOP)
    self.w_cond = []
    self.reset()
  
  def reset(self):
    for condfr in self.w_cond:
      condfr.pack_forget()
    self.w_cond = []
    self.type.set(self.object.type)
    for cond in self.object.conditions:
      self.addCond(cond)
    if len(self.w_cond) == 0:
      self.addCond()
  
  def addCond(self, cond=None):
    if cond is None:
      cond = Condition()
    self.w_cond.append(ConditionEditor(self.csfr, cond))
    self.w_cond[-1].pack(side=TOP, fill=X, expand=1, padx=5)
  
  def delCond(self):
    if len(self.w_cond) > 1:
      self.w_cond[-1].pack_forget()
      del self.w_cond[-1]
  
  def apply(self):
    self.object.conditions = []
    for cond in self.w_cond:
      self.object.add(cond.get())
    self.object.type = self.type.get()
  
  def get(self):
    self.apply()
    return self.object

class ConditionEditor(CoinObjectEditor):
  
  def body(self):
    targetlist = POST_FIELDS.copy()
    targetlist['trib'] = "Tribune"
    self.w_target = ComboField(self._fr, '', targetlist, self.object.target)
    self.w_target.pack(side=LEFT, anchor=W)
    self.w_target.bind_onchange(self.fill_op)
    self.w_op = self.w_arg = self.w_cset = None
    self.cur_op = self.object.op
    self.cur_arg = self.object.expr
    self.fill_op()
  
  def fill_op(self):
    if self.w_op is not None:
      self.w_op.pack_forget()
    if self.w_arg is not None:
      self.cur_arg = self.w_arg.get()
    op_list = [ OP_MATCHES, OP_NOT_MATCHES ]
    target = self.w_target.get()
    if target in [ 'id', 'troll', 'time' ]:
      op_list += [ OP_GREATER, OP_LESS ]
    if target in [ 'login', 'info' ]:
      op_list += [ OP_CURRENT_ID, OP_ONE_ID ]
    if target == 'message':
      op_list += [ OP_HAS_REF_TO ]
    tmpops = {}
    for op in op_list:
      tmpops[op] = op_descr[op]
    self.w_op = ComboField(self._fr, '', tmpops)
    self.w_op.pack(side=LEFT, anchor=CENTER)
    if self.cur_op in op_list:
      self.w_op.set(self.cur_op)
    self.w_op.bind_onchange(self.fill_arg)
    self.fill_arg()
  
  def fill_arg(self):
    if self.w_arg is not None:
      self.w_arg.pack_forget()
    if self.w_cset is not None:
      self.w_cset.pack_forget()
    self.w_arg = self.w_cset = None
    self.cur_op = self.w_op.get()
    if self.cur_op in [ OP_MATCHES, OP_NOT_MATCHES, OP_GREATER, OP_LESS ]:
      self.w_arg = TextField(self._fr, '', self.cur_arg)
      self.w_arg.pack(side=RIGHT, anchor=E)
    elif self.cur_op == OP_HAS_REF_TO:
      self.w_cset = Frame(self._fr)
      Label(self.w_cset, text="post vrifiant :").pack(side=LEFT)
      Button(self.w_cset, text="...", command=self.open_cset).pack(side=LEFT)
      self.w_cset.pack(side=LEFT)
  
  def open_cset(self):
    if self.object.cset is None:
      self.object.cset = ConditionSet()
    CSetDialog(self._fr, self.object.cset)
  
  def apply(self):
    self.object.target = self.w_target.get()
    self.object.op = self.w_op.get()
    if self.w_arg is not None:
      self.object.expr = self.w_arg.get()
  

## -----------------------------------------------
## Editeur d'options
## -----------------------------------------------

class OptionsEditor(CoinObjectEditor):

  def body(self):
    self.opt_set = self.object.val('toedit')
    self.widgets = {}
    
    if len(self.opt_set) > 1:
      self.config(relief=FLAT)
      self.tab = Pmw.NoteBook(self._fr)
      self.tab.pack(fill=BOTH, expand=1)
    else:
      self.tab = self._fr
    
    for categ in self.opt_set.keys():
      if len(self.opt_set) > 1:
        fr = self.tab.add(categ)
      else:
        Label(self.tab, text="Paramtres - "+categ, background=COLOR_TITLE).pack(fill=X)
        fr = Frame(self.tab)
        fr.pack(fill=BOTH, padx=10, pady=10)
      opt_list = [ (option, self.object.get(option)) for option in self.opt_set[categ] ]
      for name, option in opt_list:
        if option.type == TYPE_BOOL:
          self.disp_bool(fr, name, option)
        elif option.type == TYPE_INT:
          self.disp_int(fr, name, option)
        elif option.type == TYPE_LIST:
          self.disp_list(fr, name, option)
        elif option.type == TYPE_CHOICE:
          self.disp_choice(fr, name, option)
        elif option.type == TYPE_CHOICE_MULTI:
          self.disp_choice_multi(fr, name, option)
        elif option.type == TYPE_FILE:
          self.disp_file(fr, name, option)
        elif option.type == TYPE_COLOR:
          self.disp_color(fr, name, option)
        else:
          self.disp_other(fr, name, option)
    
    if len(self.opt_set) > 1:
      self.tab.setnaturalsize()
      if self.opt_set.has_key("Gnral"):
        self.tab.selectpage("Gnral")
  
  def disp_bool(self, fr, name, option):
    self.widgets[name] = BoolField(fr, option.descr, value=option.get())
    self.widgets[name].pack(pady=2, anchor=W)
  
  def disp_int(self, fr, name, option):
    self.widgets[name] = IntField(fr, option.descr, value=option.get())
    self.widgets[name].pack(pady=2, fill=X)
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    self.widgets[name] = OrderedListBoxWithButtons(fr, option.get(), width=40, height=10)
    self.widgets[name].pack(fill=X)
  
  def disp_choice(self, fr, name, option):
    if name == 'log_level':
      items = [ "CRITICAL", "ERROR", "WARNING", "INFO", "DEBUG" ]
    elif name == 'default_id':
      items = self.extra_arg.pycc.identities.get().keys()
    elif name == 'slip_style':
      items = [ "Tags encoded", "Tags not encoded", "Hybride" ]
    elif name == 'srv_type':
      items = [ "templeet", "daCode" ]
    else:
      items = []
    self.widgets[name] = ComboField(fr, option.descr, items, value=option.get())
    self.widgets[name].pack(pady=2, fill=X)
  
  def disp_choice_multi(self, fr, name, option):
    if name == 'startup_bouchots':
      items = get_available_bouchots(self.extra_arg)
    elif name == 'startup_modules':
      items = self.extra_arg.mod_list
    elif name == 'disp_post_attrs':
      items = available_attrs # self.extra_arg.pycc.options.val('available_attrs')
    else:
      items = []
    self.widgets[name] = MultiChoiceField(fr, option.descr, items, value=option.get(), maxbycol=9)
    self.widgets[name].pack(fill=X)
  
  def disp_file(self, fr, name, option):
    self.widgets[name] = FileField(fr, option.descr, value=option.get())
    self.widgets[name].pack(pady=2, fill=X)
  
  def disp_color(self, fr, name, option):
    self.widgets[name] = ColorField(fr, option.descr, value=option.get())
    self.widgets[name].pack(pady=2, fill=X, anchor=E)
  
  def disp_other(self, fr, name, option):
    self.widgets[name] = TextField(fr, option.descr, value=option.get())
    self.widgets[name].pack(pady=2, fill=X)
  
  def bind_buttons(self, button):
    return self.reset
  
  def reset(self):
    for opt in self.widgets.keys():
      self.widgets[opt].reset()
  
  def apply(self):
    for opt in self.widgets.keys():
      # if self.object.get(opt).categ in self.opt_set.keys():
      if self.object.get(opt).type == TYPE_LIST:
        self.object.set(opt, self.widgets[opt].getAll())
      else:
        self.object.set(opt, self.widgets[opt].get())
    self.object.save()
  
