# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module pikaplop
##   Ce module dfinit les plug-ins transformeurs de texte suivants:
##   - Plopifier (un plopificateur imitant celui de wmcoincoin)
##   - Pikachizer (traduction en langage pikachu)
##   - SMSifier (traduction en langage SMS kipu)
##   - Babelfish (interface vers babelfish, nombreuses traductions possibles)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from coinlib.parsers import zapTags
from coinlib.persistants import Option, OptionSet
from cointklib.widgets import OptionsEditor
from cointklib.basicwidgets import *

import string, re, urllib
from random import randrange, choice

def build_plugins(pycc):
  return { \
    'plopifier': Plopifier(pycc, "Plopifier", "Traduit le texte en plop meuh de gruiiik limule ka-pika"),
    'pikachize': Pikachizer(pycc, "Pikachizer", "Traduit le texte en langage Pikachu"),
    'sms': SMSifier(pycc, "SMSifier", "Traduit le texte en langage SMS (arrrgh)"),
    'babelfish': Babelfish(pycc, "Babelfish", "Traduit le texte en diverses langues via babelfish")
  }


class PlopOptions(OptionSet):
  def setName(self, name):
    self.name = "Plopifier"
  
  def setObjects(self):
    self._objects = {
      'plop_words': Option("Liste des mots-plop", value=[ 
          'pl0p', 'co1n', 'pika', 'plip', 'zzZz', 'burp',
          'meuuh', 'kikoo', 'pouet', 'prout', 'gaaaa', 'moule', 'miaou',
          'plooop', 'gnegne', 'gargle', 'blabla', 'limule', 'gloups',
          'nartaaa', 'gruiiik', 'blouark', 'flurk', 'ka-pika',
          'glouglou', 'kapou', 'buarrgll', 'coincoin',
          'fmelebele', 'mollusque', 'coccimule', 'pschiittt',
          'croucroute', 'piiikachuu', 'bzzonnng',
          'rastakoure', 'wolfenstein', 'porteninwak',
          'britneycdkey', 'super-tomate', 'eviv-bulgroz',
          'obiwan-kenobi', 'bananemagique'
        ], type=TYPE_LIST),
      'plop_density': Option("Densit de la plopification (en %)", value=75, type=TYPE_INT),
      'plop_exclude': Option("Expressions  ne pas plopifier", value="^.*([0-9=:]+|\[url\]).*$"),
      'toedit':    Option("Editables", { \
          "Gnral": [ 'plop_words', 'plop_density', 'plop_exclude' ]
        } )
    }

class PlopifierEditor(OptionsEditor):
  
  def body(self):
    OptionsEditor.body(self)
    fr = Frame(self.tab)
    self.plopword = TextField(fr, "Nouveau mot-plop:")
    self.plopword.pack(side=LEFT)
    Button(fr, text="Ajouter", command=self.new).pack(side=LEFT)
    fr.pack(anchor=W)
  
  def disp_list(self, fr, name, option):
    Label(fr, text=option.descr).pack(pady=2)
    self.widgets[name] = SortedListBox(fr, option.value, width=15, height=10)
    self.widgets[name].pack(fill=X)
    # self.widgets[name].bind_new(self.new)
  
  def new(self):
    plopword = self.plopword.get()
    if not plopword:
      return
    self.widgets['plop_words'].add(plopword)
  
  def apply(self):
    for opt, widget in self.widgets.items():
      if opt == 'plop_words':
        self.object.set(opt, widget.getAll())
      else:
        self.object.set(opt, widget.get())
    self.object.save()

class Plopifier(TextPlugin):
  
  def init(self):
    self.plop_exclude = re.compile(self.options.val('plop_exclude'))
  
  def load_config(self):
    self.options = PlopOptions(path=self.config_path)
  
  def config_frame(self, master):
    return PlopifierEditor(master, self.options, self.pycc)
  
  def action_menu(self):
    self.pycc.showConfig('cplug_plopifier')
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, font=('Verdana', 8, 'italic'), foreground='#a0a0a0')
  
  def formap(self, item):
    res = item
    if not self.plop_exclude.match(item):
      if randrange(100) < self.options.val('plop_density'):
        list = [w for w in self.options.val('plop_words') if len(w) == len(item)]
        if len(list) > 0:
          res = choice(list)
    return res
  
  def text_transform(self, text, tags={}):
    return " ".join(map(self.formap, text.split()))
  

class Pikachizer(TextPlugin):
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, foreground='#f0f000')
  
  def text_transform(self, text, tags={}):
    try:
      texte = text.encode('utf-8')
    except:
      print "## unicodage ratai"
      texte = ""
    if not text:
      try:
        texte = text.encode('iso-8859-15')
      except:
        texte = text
        print "## unicodage ratai 2"
    try:
      query = urllib.urlencode({'text': texte})
    except:
      print "## urlencode ratai"
      query = "text="+texte
    try:
      f = open_url("pikachize.eye-of-newt.com", "/pika.cgi?%s" % query, self.pycc.options)
      res = f.read()
      f.close()
      start = res.find("<PRE>") +5
      end = res.find("</PRE>", start)
      return res[start:end]
    except:
      self.error = "Pikachization impossible (erreur rseau ou du serveur pikachize)"
  

class SMSifier(TextPlugin):
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, foreground='#f000f0')
  
  def init(self):
    self.taux = 100 # AFR: customisable
  
  def text_transform(self, text, tags={}):
    try:
      text = text.encode('utf-8')
    except:
      log.warning("%s: chane d'entre non utf8-isable" % self.name)
    try:
      query = urllib.urlencode({'proba': self.taux, 'text': text})
      f = open_url("glossa.fltr.ucl.ac.be", "/programs/fr2sms.pl", self.pycc.options, data=query, headers={'Accept-Charset': "ISO-8859-1,utf-8;q=0.7,*;q=0.7"})
      page = f.read()
      f.close()
    except:
      self.error = "SMSification impossible : Erreur rseau ou du serveur"
      return
    start = page.find("URL=http://") + 11
    end = page.find('"', start)
    res = page[start:end]
    delim = res.find('/')
    host = res[:delim]
    url = res[delim:]
    try:
      f = open_url(host, url, self.pycc.options, data=query, headers={'Accept-Charset': "ISO-8859-1,utf-8;q=0.7,*;q=0.7"})
      page = f.read()
      f.close()
    except:
      self.error = "SMSification impossible : Erreur rseau ou du serveur"
      return
    start = page.find('<div align="left" style="overflow:scroll;height:245px;width:380px;">') + 68
    end = page.find("</div>", start)
    res = page[start:end].strip()
    return zapTags(res)
  

class Babelfish(TextPlugin):
  
  def init(self):
    self.lang = "fr_en"
    self.available_langs = [ "zh_en", "zt_en", "en_zh", "en_zt", "en_nl", "en_fr", 
                             "en_de", "en_el", "en_it", "en_ja", "en_ko", "en_pt",
                             "en_ru", "en_es", "nl_en", "nl_fr", "fr_en", "fr_de",
                             "fr_el", "fr_it", "fr_pt", "fr_nl", "fr_es", "de_en", 
                             "de_fr", "el_en", "el_fr", "it_en", "it_fr", "ja_en",
                             "ko_en", "pt_en", "pt_fr", "ru_en", "es_en", "es_fr" ]
  
  def get_style(self):
    return Style("[plugin] %s" % self.name, foreground='#00f0f0')
  
  def text_transform(self, text, tags={}):
    if self.lang not in self.available_langs:
      self.error = "%s: '%s' n'est pas une combinaison de langues valide" % (self.name, self.lang)
      return text
    try:
      text = text.encode('utf-8')
    except:
      log.warning("%s: chane d'entre non utf8-isable" % self.name)
    try:
      query = urllib.urlencode({'lp': self.lang, 'trtext': text, 'doit': "done", 'intl': "1", 'tt': "urltext"})
      print "### Query", query
      f = open_url("babelfish.altavista.com", "/tr", self.pycc.options, data=query, headers={'Accept-Charset': "ISO-8859-1,utf-8;q=0.7,*;q=0.7", 'Referer': "http://babelfish.altavista.com/tr"})
      page = f.read()
      f.close()
    except:
      self.error = "%s: Erreur rseau ou du serveur" % self.name
      print "### Error Traducteur", self.error
      return
    try:
      res = unicode(page, "utf-8")
    except:
      res = page
    if res.find(ur"Translation Error</title>") > 0:
      start = res.find(ur"<b><i>") + 6
      end = res.find(ur"</i></b>", start)
      return "<tt>Translation Error (%s)</tt>" % res[start:end]
    start = res.find(ur"<td bgcolor=white class=s><div style=padding:10px;>") + 51
    end = res.find(ur"</div>", start)
    return res[start:end].strip()
  
