# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module pyc2
##   Interface de PyCoinCoin.
## ----------------------------------------------------------------------

from Tkinter import *
import tkMessageBox
import Pmw
import gc, time

import logging
log = logging.getLogger('pycc')

from constants import *
from coinlib.utils import *
from coinlib.persistants import *
from pyc4.ControlCenter import ControlCenter
from pyc4.wizards import BouchotConfigWizard, BouchotUpdateWizard
from pyc4.help import About
from palmi import Palmipede
from bouchot import Bouchot
from cointklib.dialogs import LoginDialog, LogoutDialog


def import_modules(script_path='.'):
  """ Fonction qui repre les modules dipsonibles et construit
      dynamiquement un hash nom_module => fonction de chargement du module
  """
  log.debug("## Entering import_modules")
  basepath = os.path.join(script_path, "modules")
  modules = {}
  modlist = [ file for file in os.listdir(basepath) if os.path.isdir(os.path.join(basepath, file)) ]
  for module in modlist:
    modpath = 'modules.'+module+'.'+module
    # AFR: gestion exception ?
    #try:
    __import__(modpath)
    mod = sys.modules[modpath]
    modules[module] = mod.build_module
    log.info("Module '%s' import" % module)
    #except:
    #  log.error("Impossible d'importer le module '%s'" % module)
  return modules


def import_plugins(script_path, pycc):
  """ Fonction qui repre les plugins dipsonibles et construit
      dynamiquement un hash nom_plugin => fonction de chargement du plugin
  """
  log.debug("## Entering import_plugins")
  basepath = os.path.join(script_path, "plugins")
  plugins = {}
  pluglist = [ file[:-3] for file in os.listdir(basepath) if file[-3:] == '.py' and file != "__init__.py" ]
  for plugin in pluglist:
    plugpath = 'plugins.'+plugin
    try:
      del(sys.modules[plugpath])
    except:
      pass
    #try:
    __import__(plugpath)
    mod = sys.modules[plugpath]
    for plugname, plugobj in mod.build_plugins(pycc).items():
      plugins[plugname] = plugobj
    log.info("Plugin '%s' import" % plugin)
    #except:
    #  log.error("Impossible d'importer le plugin '%s'" % plugin)
  return plugins


## ----------------------------------------------------------------
## Configuration des vnements clavier / souris / menu contextuel
## ----------------------------------------------------------------

#== Evnements clavier par dfaut
key_bindings = {
  'Return': PyccEvent(ActionInsert("%h")),
  'Alt-w':  PyccEvent(ActionSetStyle("myposts"))
}
class PinniKeySet(Writeable):
  def __init__(self, name="PinniKeys", path="config", fileName=None):
    Writeable.__init__(self, name, path, fileName)
    if not self.load():
      for keystroke, keyevent in key_bindings.items():
        self.add(keystroke, keyevent)

#== Evnements souris par dfaut
click_bindings = {
  'Control-ButtonRelease-2': PyccEvent(ActionSetStyle("myposts", target="time"))
}
class PinniClickSet(Writeable):
  def __init__(self, name="PinniClicks", path="config", fileName=None):
    Writeable.__init__(self, name, path, fileName)
    if not self.load():
      for click, clickevent in click_bindings.items():
        self.add(click, clickevent)

#== Menu contextuel par dfaut
default_menu = {
  "Copier l'horloge":    PyccEvent(ActionInsert("%h")),
  "Marquer comme  moi": PyccEvent(ActionSetStyle("myposts"))
}
class PinniMenuSet(Writeable):
  def __init__(self, name="PinniMenu", path="config", fileName=None):
    Writeable.__init__(self, name, path, fileName)
    if not self.load():
      for entry, menuevent in default_menu.items():
        self.add(entry, menuevent)


## ----------------------------------------------------------------
## L'objet - interface du canard.
## ----------------------------------------------------------------

class PyCoinCoin(object):
  """ La fentre principale de pycc.
  """
  def __init__(self, master, config_path, log_level=None, script_path='.', blist=[]):
    self.init = 1
    self.root = master   # La root window de l'appli
    self.root.protocol("WM_DELETE_WINDOW", self.panpan)
    self.config_path = config_path
    self.log_level = log_level
    self.script_path = script_path
    self.system_color = self.root.cget('background')
    self.balloon = Pmw.Balloon(master)
    self.balloon.configure(state='balloon')
    self.pycccc = None
    self.initialize()
    self.build_interface()
    
    toload = get_available_bouchots(self.config_path)
    if toload:
      for bouchal in toload:
        self.add_bouchot(bouchal)
    else:
      # Aucun bouchot dtect: on lance le wizard
      self.new_bouchot()
    
    arglist = [ b for b in blist if b in self.bouchots.keys() ]
    if len(arglist) > 0:
      self.startup_bouchots = arglist
    
    for bouchal in self.startup_bouchots:
      if bouchal in self.bouchots.keys():
        self.load_bouchot(bouchal)
    
    self.init = 0
    if not self.startup_bouchots:
      self.cur_bouchot = None
      log.error("Aucun bouchot n'a dmarr...")
    else:
      self.select_tab(self.get_bouchot())
      self.tab.setnaturalsize()
    
    self.disp_plugin_menu()
  
  def initialize(self):
    """ Initialisation de pycc - variables internes, etc. """
    log.debug("## Entering PyCoinCoin.init")
    self.load_config()
    self.apply_config('cgen')
    log.debug("Bouchots  dmarrer: " + str(self.startup_bouchots))
    self.urllist = {}
    self.images = { \
      'minus': PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'EYyPoivG614LAlg7ZZbxoR8UADs='),
      'plus':  PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'FIyPoiu2sJyCyoF7W3hxz850CFIA\nADs=')
    }
    self.bouchots = {}
    self.clipboard = ""
    self.load_modules()
    self.load_plugins()
    self.processor.start()
  
  def load_modules(self):
    log.debug("## Entering PyCoinCoin.load_modules")
    self.modules = import_modules(self.script_path)
  
  def load_plugins(self):
    log.debug("## Entering PyCoinCoin.load_plugins")
    self.plugins = import_plugins(self.script_path, self)
  
  def reload_plugins(self):
    log.debug("## Entering PyCoinCoin.reload_plugins")
    self.setStatus('Rechargement des plug-ins en cours...')
    self.load_plugins()
    self.disp_plugin_menu()
    gc.collect()
    self.setStatus('Plug-ins mis  jour!')
  
  # -----------------------------------------
  # Mthodes de gestion de l'interface
  # -----------------------------------------
  
  def build_interface(self):
    """ Construit l'interface du coincoin.  
    """
    master = self.root
    
    #== La status bar :
    self.status = Pmw.MessageBar(master, entry_background=self.root.cget('background'))
    self.status.pack(side=BOTTOM, fill=X)
    
    #== Les bulles d'aide
    self.balloon = Pmw.Balloon(master)
    self.balloon.configure(statuscommand=self.status.helpmessage)
    
    #== La barre de menus
    self.menubar = Pmw.MainMenuBar(master, balloon=self.balloon, hotkeys=0)
    self.root.configure(menu=self.menubar)
    
    #- Menu Canard
    self.menubar.addmenu('Canard', '', traverseSpec=0)
    self.menubar.addmenuitem('Canard', 'command', 'Configurer un nouveau bouchot',
      label="Nouveau bouchot...", command=self.new_bouchot)
    self.menubar.addmenuitem('Canard', 'separator')
    self.menubar.addmenuitem('Canard', 'command', "S'authentifier le bouchot courant",
      label="Log on bouchot", command=self.logon_bouchot)
    self.menubar.addmenuitem('Canard', 'command', "Se deloguer du bouchot courant",
      label="Logout from bouchot", command=self.logout_bouchot)
    self.menubar.addmenuitem('Canard', 'command', 'Recharger le bouchot courant',
      label="Recharger bouchot", command=self.reload_bouchot)
    self.menubar.addmenuitem('Canard', 'command', 'Fermer le bouchot courant',
      label="Fermer bouchot", command=self.kill_bouchot)
    self.menubar.addmenuitem('Canard', 'separator')
    self.menubar.addmenuitem('Canard', 'command', 'Quitter pyCoinCoin',
      label="Pan Pan !", command=self.panpan)
    
    #- Menu Bouchot
    # La liste bouchots disponibles
    self.menubar.addmenu('Bouchots', '', traverseSpec=0)
    self.toggleVar = {}
    
    #- Menu Configuration
    self.menubar.addmenu('Configuration', '', traverseSpec=0)
    self.menubar.addmenuitem('Configuration', 'command', 'Mise  jour des paramtres du bouchot courant',
      label="MAJ bouchot...", command=self.maj_bouchot)
    self.menubar.addmenuitem('Configuration', 'command', 'Configuration des paramtres du bouchot courant',
      label="Config bouchot...", command=self.showBouchotConfig)
    self.menubar.addmenuitem('Configuration', 'command', 'Configuration des styles, couleurs... du bouchot courant',
      label="Apparence bouchot...", command=lambda s=self: s.showBouchotConfig('clook'))
    self.menubar.addmenuitem('Configuration', 'command', 'Configuration des modules du bouchot courant',
      label="Modules...", command=lambda s=self: s.showBouchotConfig('cmod'))
    self.menubar.addmenuitem('Configuration', 'separator')
    
    self.menubar.addmenuitem('Configuration', 'command', 'Gestion de vos diffrentes identits',
      label="Identits...", command=lambda s=self: s.showConfig('cident'))
    self.menubar.addmenuitem('Configuration', 'command', 'Actions  effectuer lors de la rception des posts',
      label="Da Post Processor...", command=lambda s=self: s.showConfig('cpp'))
    self.menubar.addmenuitem('Configuration', 'command', 'Configuration du Pinnipde Teletype',
      label="Pinnipde...", command=lambda s=self: s.showConfig('cpinni'))
    self.menubar.addmenuitem('Configuration', 'command', 'Configuration du Palmipde Editor',
      label="Palmipde...", command=lambda s=self: s.showConfig('cpalmi'))
    # AFR dans le futur :)
    #self.menubar.addmenuitem('Configuration', 'command', 'Gestion des bots',
    #  label="Da Bots Manager...", command=lambda s=self: s.showConfig('cbot'))
    
    self.menubar.addmenuitem('Configuration', 'separator')
    self.menubar.addmenuitem('Configuration', 'command', 'Centre de contrle pour tous les paramtres de pyCoinCoin',
      label="<PyCC Control Center>", command=self.showConfig, traverseSpec='y')
    
    #- Menu plugins
    self.menubar.addmenu('Plugins', '', traverseSpec=0)
    
    #- Menu Help
    self.menubar.addmenu('Aide', '', name='help', traverseSpec=0)
    self.menubar.addmenuitem('Aide', 'command', "Affiche le guide d'installation et d'utilisation de pyCoinCoin",
      label="Guide d'utilisation", command=self.nothing)
    self.menubar.addmenuitem('Aide', 'command', "Aide sur les modules disponibles, leur configuration",
      label="Modules", command=self.nothing)
    self.menubar.addmenuitem('Aide', 'command', "Quels plug-ins sont disponibles et comment les utiliser",
      label="Plug-ins", command=self.nothing)
    self.menubar.addmenuitem('Aide', 'command', "Documentation des APIs de pyCoinCoin, pour les dveloppeurs",
      label="Dveloppement", command=self.nothing)
    self.menubar.addmenuitem('Aide', 'separator')
    self.menubar.addmenuitem('Aide', 'command', "Renseignements divers sur pyCoinCoin",
      label="A propos...", command=lambda w=self: About(w))
    
    #== Le Panel permettant de redimensionner le tout. (JB)
    self.leftpan = Pmw.PanedWidget(self.root, orient='vertical')
    self.leftpan.pantop =self.leftpan.add('top', min=200)
    self.leftpan.panbottom = self.leftpan.add('bottom', min=100)
    self.leftpan.pack(fill=BOTH, expand=1)
    
    #== La fentre  onglets
    self.tab = Pmw.NoteBook(self.leftpan.pantop,
            raisecommand=self.select_tab, hull_width=400, hull_height=50 )
    self.tab.pack(fill=BOTH, expand=1)
    
    #== Le Palmipde Editor
    self.palmi = Palmipede(self.leftpan.panbottom, self)
    self.palmi.pack(fill=BOTH, expand=1)
    self.palmi.bindKeys()
    
    #== Le popup
    self.popup = Text(master, wrap=WORD, relief=RAISED)
    self.popup_visible = 0
    
    #== Le menu contextuel
    self.ctxt_menu = Menu(self.root, tearoff=0)
  
  def menu_add(self, bouchal):
    log.debug("## Entering PyCoinCoin.menu_add(%s)" % bouchal)
    self.toggleVar[bouchal] = IntVar()
    self.menubar.addmenuitem('Bouchots', 'checkbutton',
      'Ouvrir/Fermer le bouchot "%s"' % bouchal,
      label=bouchal, command=lambda s=self, t=bouchal: s.toggleBouchot(t),
      variable=self.toggleVar[bouchal])
  
  def disp_plugin_menu(self):
    self.menubar.deletemenuitems('Plugins', 0, 100)
    self.menubar.addmenuitem('Plugins', 'command', "Recharge les plug-ins sans qu'il soit ncessaire de redmarrer pyCoinCoin",
      label="Recharger les plug-ins", command=self.reload_plugins)
    self.menubar.addmenuitem('Plugins', 'separator')
    categs = []
    for plugin in self.plugins.values():
      if not plugin.categ:
        self.menubar.addmenuitem('Plugins', 'command', plugin.descr,
          label=plugin.name, command=plugin.action_menu)
      else:
        if plugin.categ not in categs:
          try:
            self.menubar.deletemenu(plugin.categ)
          except:
            pass
          self.menubar.addcascademenu('Plugins', plugin.categ)
          categs.append(plugin.categ)
        self.menubar.addmenuitem(plugin.categ, 'command', plugin.descr,
          label=plugin.name, command=plugin.action_menu)
  
  def colorize_tab(self, bouchal, color):
    log.debug("## Entering PyCoinCoin.colorize_tab(%s, %s)" % (bouchal, color))
    self.tab._pageAttrs[bouchal]['tabbutton'].config(background=color)
  
  def blink_tab(self, bouchal, color=STATE_BLINK, timer=500):
    try:
      butwidget = self.tab._pageAttrs[bouchal]['tabbutton']
      current = butwidget.cget('background')
      if current == color:
        ncolor = self.bouchots[bouchal].cur_color
      else:
        self.bouchots[bouchal].cur_color = current
        ncolor = color
      if self.bouchots[bouchal].blinking:
        self.colorize_tab(bouchal, ncolor)
        if timer:
          butwidget.after(timer, lambda s=self, b=bouchal, c=color, t=timer: s.blink_tab(b,c,t))
      else:
        self.colorize_tab(bouchal, self.system_color)
    except TclError:
      log.warning("blink_tab ratai, Tcl a plantai")
  
  def select_tab(self, bouchal):
    log.debug("## Entering PyCoinCoin.select_tab(%s)" % bouchal)
    if self.init: return
    if bouchal not in self.bouchots: return
    self.cur_bouchot = bouchal
    try:
      apply_styles(self.popup, self.bouchots[bouchal].styles, 'popup', self.default_styles)
    except KeyError:
      pass
    self.palmi.set_trib(bouchal)
    self.tab.selectpage(bouchal)    # Slection de l'onglet
    self.colorize_tab(bouchal, self.system_color)
    self.bouchots[bouchal].blinking = 0
  
  def nextTab(self):
    list = self.tab.pagenames()
    idx = list.index(self.get_bouchot())
    self.select_tab(list[(idx+1)%len(list)])
  
  def prevTab(self):
    list = self.tab.pagenames()
    idx = list.index(self.get_bouchot())
    self.select_tab(list[(idx-1)%len(list)])
  
  def setStatus(self, message):
    # log.debug("## Entering PyCoinCoin.setStatus(%s)" % message)
    self.status.message('state', message)
    # self.root.update_idletasks()
  
  # -----------------------------------------
  # Mthodes de gestion de la config
  # -----------------------------------------
  
  def load_config(self):
    log.info("Chargement des paramtres de configuration...")
    self.options        = GlobalOptions(path=self.config_path)
    self.identities     = Identities(path=self.config_path)
    self.default_styles = PinniStyleSet(path=self.config_path)
    self.pinni_keys     = PinniKeySet(path=self.config_path)
    self.pinni_clicks   = PinniClickSet(path=self.config_path)
    self.pinni_menu     = PinniMenuSet(path=self.config_path)
    self.processor      = PostProcessor(path=self.config_path)
  
  def save_config(self):
    log.debug("## Entering PyCoinCoin.save_config")
    self.options.save()
    self.identities.save()
    self.default_styles.save()
    self.pinni_keys.save()
    self.pinni_clicks.save()
    self.pinni_menu.save()
    self.processor.save()
  
  def showConfig(self, section="root", parent=None):
    log.debug("## Entering PyCoinCoin.showConfig(%s)" % section)
    if parent is None: parent = self.root
    if self.pycccc is None:
      self.pycccc = ControlCenter(parent, self, section)
    else:
      self.pycccc.tree.expand(section)
      self.pycccc.tree.select(section)
      self.pycccc.focus_set()
  
  def showBouchotConfig(self, section=''):
    log.debug("## Entering PyCoinCoin.showBouchotConfig(%s)" % section)
    bsec = self.get_bouchot()
    if section:
      bsec += "_"+section
    self.showConfig(bsec)
  
  def apply_config(self, section):
    log.debug("## Entering PyCoinCoin.apply_config(%s)" % section)
    # AFR section plugins
    ignore_sections = ['root', 'cekeys', 'cpkeys', 'cpclic', 'cpmenu', 'cpp', 'cbou']
    if section in ignore_sections:
      return
    
    if section == 'cgen':
      if self.log_level is None:
        set_log_level(log, self.options.val('log_level'))
      if self.options.val('startup_mode'):
        self.startup_bouchots = self.options.val('lastses_bouchots')
      else:
        self.startup_bouchots = self.options.val('startup_bouchots')
      self.options.save()
    
    elif section == 'cident':
      self.palmi.update_id_list()
      self.identities.save()
    
    elif section == 'cpalmi':
      self.palmi.apply_config()
      self.palmi.options.save()
    
    elif section == 'celook':
      self.palmi.apply_styles()
      self.palmi.styles.save()
    
    elif section == 'cpinni':
      pass # Pour le moment, rien  faire
    
    elif section == 'cplug':
      pass # AFR: pour les plugins
    
    elif section[:6] == 'cplug_':
      pass # Conf spcifique plugins
    
    elif section =='cbot':
      pass # AFR: pour les bots
    
    elif section == 'cplook':
      for bouchal in self.bouchots.values():
        if bouchal.trib is not None:
          if bouchal.trib.loaded:
            bouchal.trib.apply_styles()
      self.default_styles.save()
    
    elif section.find("_") > 0:
      name, subsec = section.split("_")
      bouchot = self.bouchots[name]
      if not bouchot.started:
        return
      if subsec == 'clook':
        if bouchot.trib is not None:
          if bouchot.trib.loaded:
            bouchot.apply_styles()
      elif subsec == 'cmod':
        pass # Rien  faire
      elif subsec == 'cplug':
        pass # AFR pour plugins
      else: # subsec == un module particulier
        module = bouchot.modules[subsec]
        if module.loaded:
          module.apply_config()
    
    else: # section == un nom de bouchot
      bouchot = self.bouchots[section]
      if bouchot.started:
        bouchot.apply_config()
  
  # -----------------------------------------
  # Mthodes de gestion des bouchots
  # -----------------------------------------
  
  def load_bouchot(self, bouchal):
    log.debug("## Entering PyCoinCoin.load_bouchot(%s)" % bouchal)
    page = self.tab.add(bouchal)
    self.balloon.bind(self.tab._pageAttrs[bouchal]['tabbutton'], "Bouchot: %s" % bouchal)
    self.tab.selectpage(bouchal)
    self.cur_bouchot = bouchal
    self.bouchots[bouchal].build_interface(page)
    self.bouchots[bouchal].start()
    self.toggleVar[bouchal].set(1)
    self.palmi.update_trib_list()
    self.palmi.set_trib(bouchal)
  
  def add_bouchot(self, bouchal):
    log.debug("## Entering PyCoinCoin.add_bouchot(%s)" % bouchal)
    self.bouchots[bouchal] = Bouchot(self, bouchal)
    self.menu_add(bouchal)
  
  def new_bouchot(self):
    log.debug("## Entering PyCoinCoin.new_bouchot")
    self.canceled = 1
    self.newbouchot = ''
    BouchotConfigWizard(self, self.root)
  
  def maj_bouchot(self):
    log.debug("## Entering PyCoinCoin.maj_bouchot")
    self.canceled = 1
    BouchotUpdateWizard(self, self.root, self.cur_bouchot)
  
  def logon_bouchot(self, bouchal=None):
    log.debug("## Entering PyCoinCoin.logon_bouchot(%s)" % bouchal)
    if bouchal is None:
      bouchal = self.cur_bouchot
    id = self.bouchots[self.cur_bouchot].options.get('default_id').get()
    self.root.result = ("", "")
    LoginDialog(self.root, self, bouchal, id)
  
  def logout_bouchot(self, bouchal=None):
    log.debug("## Entering PyCoinCoin.logout_bouchot(%s)" % bouchal)
    if bouchal is None:
      bouchal = self.cur_bouchot
    id = self.bouchots[self.cur_bouchot].options.get('default_id').get()
    self.root.result = ("", "")
    LogoutDialog(self.root, self, bouchal, id)
  
  def reload_bouchot(self, bouchal=None):
    log.debug("## Entering PyCoinCoin.reload_bouchot(%s)" % bouchal)
    if bouchal is None:
      bouchal = self.cur_bouchot
    self.kill_bouchot(bouchal)
    self.load_bouchot(bouchal)
  
  def kill_bouchot(self, bouchal=None):
    log.debug("## Entering PyCoinCoin.kill_bouchot(%s)" % bouchal)
    if bouchal is None:
      bouchal = self.cur_bouchot
    if self.bouchots[bouchal].started:
      self.bouchots[bouchal].quit()
      self.toggleVar[bouchal].set(0)
      self.tab.delete(bouchal)
    self.palmi.update_trib_list()
  
  def destroy_bouchot(self, name):
    log.debug("## Entering PyCoinCoin.destroy_bouchot(%s)" % name)
    self.kill_bouchot(name)
    top = os.path.join(self.config_path, "bouchots", name)
    try:
      for root, dirs, files in os.walk(top, topdown=False):
        for fich in files:
          os.remove(os.path.join(root, fich))
        for fich in dirs:
          os.rmdir(os.path.join(root, fich))
      os.rmdir(top)
    except:
      log.error("Impossible de dtruire tous les fichiers du bouchot '%s'" %  name)
    del self.bouchots[name]
  
  def toggleBouchot(self, bouchal):
    log.debug("## Entering PyCoinCoin.toggleBouchot(%s)" % bouchal)
    # Ouvre/ferme un bouchot en fonction de l'tat du menu
    if self.toggleVar[bouchal].get():
      self.load_bouchot(bouchal)
    else:
      self.kill_bouchot(bouchal)

  def activeBouchots(self):
    log.debug("## Entering PyCoinCoin.activeBouchots")
    return [name for name, bouchal in self.bouchots.items() if bouchal.started]
  
  def get_bouchot(self):
    """ Renvoie le bouchot courant """
    log.debug("## Entering PyCoinCoin.get_bouchot")
    return self.cur_bouchot
  
  def getCurTrib(self):
    log.debug("## Entering PyCoinCoin.getCurTrib")
    bouchot = self.bouchots[self.cur_bouchot]
    if bouchot.trib is not None:
      if bouchot.trib.loaded:
        return bouchot.trib
    return None
  
  # -----------------------------------------
  # Mthodes diverses
  # -----------------------------------------
  
  def nothing(self):
    """ Ne fait rien. Pour les fonctions non encore implmentes """
    print "---- Ben oui il reste du boulot ----"
  
  def openBrowser(self, url, type=1):
    """ Ouvre l'url dans un browser """
    log.debug("## Entering PyCoinCoin.openBrowser")
    if type == 1:
      cmdline = self.options.val('custom_browser')
    else:
      cmdline = self.options.val('custom_browser_2')
    if cmdline.find('%url') >= 0:
      cmdline = cmdline.replace("%url", url)
    else:
      cmdline += " "+url
    return execute_cmd(cmdline)
  
  def addUrlInList(self, url, ref=None, read=0):
    aurl = self.getAtomicUrl(url)
    refs, uread = self.getBloub(aurl)
    if ref is not None and ref not in refs:
      refs.append(ref)
    if read:
      uread = read
    self.urllist[aurl] = (refs, uread)
    # print "Ich bin appending eine URL:", aurl, refs, uread
  
  def addUrls(self, urllist, ref):
    # print "### Add urls =", urllist
    for url in urllist:
      self.addUrlInList(url, ref)
  
  def getAtomicUrl(self, url):
    myurl = toUnicode(url)
    if myurl[-1] == u"/":
      myurl = url[:-1]
    if myurl[:5] == u"https":
      myurl = u"http"+myurl[5:]
    return myurl
  
  def setUrlRead(self, url, ref=None):
    """ Marque une url comme lue dans tous les bouchots ouverts """
    for bouchot in self.bouchots.values():
      if bouchot.trib is not None:
        if bouchot.trib.loaded:
          bouchot.trib.setUrlRead(url)
    uurl = self.getAtomicUrl(url)
    try:
      refs, read = self.urllist[uurl]
    except:
      refs, read = ([], 0)
    if ref is not None and ref not in refs:
      refs.append(ref)
    self.urllist[uurl] = (refs, 1)
  
  def getBloub(self, url):
    uurl = self.getAtomicUrl(url)
    try:
      refs, read = self.urllist[uurl]
    except:
      refs, read = ([], 0)
    return (refs, read)
  
  def isUrlBloub(self, url):
    return self.getAtomicUrl(url) in self.urllist.keys()
  
  def isUrlRead(self, url):
    uurl = self.getAtomicUrl(url)
    try:
      uref, read = self.urllist[uurl]
    except:
      # self.addUrlInList(url, ref)
      read = 0
    return read
  
  def getImageWithCache(self, url):
    log.debug("getImageWithCache: %s" % url)
    cacheimg = md5.new(url).hexdigest()
    imgfile = os.path.join(self.config_path, "cache", cacheimg)
    if not os.path.exists(imgfile):
      self.getImageFromUrl(url)
    return imgfile
  
  def getImageFromUrl(self, url):
    try:
      log.info("getImageFromUrl: %s" % url)
      self.setStatus("Tlchargement: %s..." % url)
      # self.status.update_idletasks()
      save_url(url, self.options, self.config_path)
      self.setStatus("Tlchargement OK")
      return 1
    except IOError:
      self.setStatus("Erreur au tlchargement de %s" % url)
      log.error("Impossible de tlcharger l'image: %s" % url)
      return 0
  
  def panpan(self):
    """ Pour quitter... """
    log.debug("## Entering PyCoinCoin.panpan")
    if self.options.val('quit_confirm'):
      if not tkMessageBox.askyesno("pyCoinCoin", "//====     ~ >x_/\nVous voulez vraiment tuer le canard ?? (esspaice de maichan !)"):
        return
    self.options.set('lastses_bouchots', self.activeBouchots())
    for bouchal in self.bouchots.keys():
      self.kill_bouchot(bouchal)
    self.palmi.kill()
    self.processor.stop()
    if self.pycccc is not None:
      self.pycccc.quit()
    self.save_config()
    self.root.destroy()

